/*
 * Copyright 2013  Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <stdio.h>
#include <errno.h>

#include <dlog.h>
#include <livebox.h>
#include <livebox-errno.h>

#include "livebox-cpp.h"
#include "livebox-impl.h"
#include "CModule.h"
#include "c-abi.h"

#define EAPI __attribute__((visibility("default")))

EAPI int livebox_initialize(const char *pkgname)
{
	CModule *module;

	module = CModule::FindModule(pkgname);
	if (module)
		return LB_STATUS_SUCCESS;

	module = CModule::Load(pkgname);
	if (!module)
		return LB_STATUS_ERROR_FAULT;

	return LB_STATUS_SUCCESS;
}

EAPI int livebox_finalize(const char *pkgname)
{
	CModule *module;

	module = CModule::FindModule(pkgname);
	if (!module)
		return LB_STATUS_ERROR_NOT_EXIST;

	module->Unload();
	return LB_STATUS_SUCCESS;
}

EAPI int livebox_create(const char *pkgname, const char *filename, const char *content, const char *cluster, const char *category)
{
	CLiveBoxImpl *box;
	CModule *module;
	int ret;

	module = CModule::FindModule(pkgname);
	if (!module)
		return LB_STATUS_ERROR_INVALID;

	box = module->FindLiveBox(filename);
	if (box)
		return LB_STATUS_ERROR_NOT_EXIST;

	ret = module->Create(filename, content, cluster, category);
	return ret;
}

EAPI int livebox_destroy(const char *pkgname, const char *filename)
{
	CModule *module;
	CLiveBoxImpl *box;
	int ret;

	module = CModule::FindModule(pkgname);
	if (!module)
		return LB_STATUS_ERROR_INVALID;

	box = module->FindLiveBox(filename);
	if (!box)
		return LB_STATUS_ERROR_NOT_EXIST;

	ret = module->Destroy(box);
	return ret;
}

EAPI int livebox_need_to_update(const char *pkgname, const char *filename)
{
	CLiveBoxImpl *box;
	CModule *module;

	module = CModule::FindModule(pkgname);
	if (!module)
		return LB_STATUS_ERROR_INVALID;

	box = module->FindLiveBox(filename);
	if (!box)
		return LB_STATUS_ERROR_NOT_EXIST;

	return box->NeedToUpdate();
}

EAPI int livebox_need_to_destroy(const char *pkgname, const char *filename)
{
	CLiveBoxImpl *box;
	CModule *module;

	module = CModule::FindModule(pkgname);
	if (!module)
		return LB_STATUS_ERROR_INVALID;

	box = module->FindLiveBox(filename);
	if (!box)
		return LB_STATUS_ERROR_NOT_EXIST;
	
	return box->NeedToDestroy();
}

EAPI int livebox_update_content(const char *pkgname, const char *filename)
{
	CLiveBoxImpl *box;
	CModule *module;

	module = CModule::FindModule(pkgname);
	if (!module)
		return LB_STATUS_ERROR_INVALID;

	box = module->FindLiveBox(filename);
	if (!box)
		return LB_STATUS_ERROR_NOT_EXIST;
	
	return box->UpdateContent();
}

EAPI int livebox_clicked(const char *pkgname, const char *filename, const char *event, double timestamp, double x, double y)
{
	CLiveBoxImpl *box;
	CModule *module;

	module = CModule::FindModule(pkgname);
	if (!module)
		return LB_STATUS_ERROR_INVALID;

	box = module->FindLiveBox(filename);
	if (!box)
		return LB_STATUS_ERROR_NOT_EXIST;

	return box->Clicked(event, timestamp, x, y);
}

EAPI int livebox_content_event(const char *pkgname, const char *filename, const char *emission, const char *source, struct event_info *event_info)
{
	CLiveBoxImpl *box;
	CModule *module;

	module = CModule::FindModule(pkgname);
	if (!module)
		return LB_STATUS_ERROR_INVALID;

	box = module->FindLiveBox(filename);
	if (!box)
		return LB_STATUS_ERROR_NOT_EXIST;

	return box->ContentEvent(emission, source, event_info);
}

EAPI int livebox_resize(const char *pkgname, const char *filename, int type)
{
	CLiveBoxImpl *box;
	CModule *module;

	module = CModule::FindModule(pkgname);
	if (!module)
		return LB_STATUS_ERROR_INVALID;

	box = module->FindLiveBox(filename);
	if (!box)
		return LB_STATUS_ERROR_NOT_EXIST;

	return box->Resize(type);
}

EAPI int livebox_change_group(const char *pkgname, const char *filename, const char *cluster, const char *category)
{
	CLiveBoxImpl *box;
	CModule *module;

	module = CModule::FindModule(pkgname);
	if (!module)
		return LB_STATUS_ERROR_INVALID;

	box = module->FindLiveBox(filename);
	if (!box)
		return LB_STATUS_ERROR_NOT_EXIST;

	return box->ChangeGroup(cluster, category);
}

EAPI int livebox_get_info(const char *pkgname, const char *filename, int *w, int *h, double *priority, char **content, char **title)
{
	CLiveBoxImpl *box;
	CModule *module;

	module = CModule::FindModule(pkgname);
	if (!module)
		return LB_STATUS_ERROR_INVALID;

	box = module->FindLiveBox(filename);
	if (!box)
		return LB_STATUS_ERROR_NOT_EXIST;

	return box->GetInfo(w, h, priority, content, title);
}

EAPI int livebox_need_to_create(const char *pkgname, const char *cluster, const char *category)
{
	CModule *module;

	module = CModule::FindModule(pkgname);
	if (!module)
		return LB_STATUS_ERROR_INVALID;

	return module->NeedToCreate(cluster, category);
}

EAPI char *livebox_pinup(const char *pkgname, const char *filename, int pinup)
{
	CLiveBoxImpl *box;
	CModule *module;

	module = CModule::FindModule(pkgname);
	if (!module)
		return NULL;

	box = module->FindLiveBox(filename);
	if (!box)
		return NULL;

	return box->PinUp(pinup);
}

EAPI int livebox_is_pinned_up(const char *pkgname, const char *filename)
{
	CLiveBoxImpl *box;
	CModule *module;

	module = CModule::FindModule(pkgname);
	if (!module)
		return NULL;

	box = module->FindLiveBox(filename);
	if (!box)
		return NULL;

	return box->IsPinnedUp();
}

EAPI int livebox_system_event(const char *pkgname, const char *filename, int event)
{
	CLiveBoxImpl *box;
	CModule *module;

	module = CModule::FindModule(pkgname);
	if (!module)
		return NULL;

	box = module->FindLiveBox(filename);
	if (!box)
		return NULL;

	return box->SystemEvent(event);
}

/* End of a file */
